"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var const_1 = require("./const");
var create_buffer_page_with_annotation_1 = __importDefault(require("./create-buffer-page-with-annotation"));
var create_buffer_root_with_acrofrom_1 = __importDefault(require("./create-buffer-root-with-acrofrom"));
var create_buffer_trailer_1 = __importDefault(require("./create-buffer-trailer"));
var get_index_from_ref_1 = __importDefault(require("./get-index-from-ref"));
var get_page_ref_1 = __importDefault(require("./get-page-ref"));
var pdf_kit_add_placeholder_1 = __importDefault(require("./pdf-kit-add-placeholder"));
var pdf_kit_reference_mock_1 = __importDefault(require("./pdf-kit-reference-mock"));
var pdfobject_1 = require("./pdfkit/pdfobject");
var read_pdf_1 = __importDefault(require("./read-pdf"));
var remove_trailing_new_line_1 = __importDefault(require("./remove-trailing-new-line"));
var plainAddPlaceholder = function (pdfBuffer, signatureOptions, signatureLength) {
    if (signatureLength === void 0) { signatureLength = const_1.DEFAULT_SIGNATURE_LENGTH; }
    var pdf = remove_trailing_new_line_1.default(pdfBuffer);
    var info = read_pdf_1.default(pdf);
    var pageRef = get_page_ref_1.default(pdf, info);
    var pageIndex = get_index_from_ref_1.default(info.xref, pageRef);
    var addedReferences = new Map();
    var pdfKitMock = {
        ref: function (input, additionalIndex, stream) {
            info.xref.maxIndex += 1;
            var index = additionalIndex != null ? additionalIndex : info.xref.maxIndex;
            addedReferences.set(index, pdf.length + 1);
            pdf = getAssembledPdf(pdf, index, input, stream);
            return new pdf_kit_reference_mock_1.default(info.xref.maxIndex);
        },
        page: {
            dictionary: new pdf_kit_reference_mock_1.default(pageIndex, {
                data: {
                    Annots: [],
                },
            }),
        },
        _root: {
            data: {},
        },
    };
    var _a = pdf_kit_add_placeholder_1.default({
        pdf: pdfKitMock,
        pdfBuffer: pdfBuffer,
        signatureLength: signatureLength,
        signatureOptions: signatureOptions,
    }), form = _a.form, widget = _a.widget;
    if (!isContainBufferRootWithAcrofrom(pdfBuffer)) {
        var rootIndex = get_index_from_ref_1.default(info.xref, info.rootRef);
        addedReferences.set(rootIndex, pdf.length + 1);
        pdf = Buffer.concat([pdf, Buffer.from('\n'), create_buffer_root_with_acrofrom_1.default(info, form)]);
    }
    addedReferences.set(pageIndex, pdf.length + 1);
    pdf = Buffer.concat([
        pdf,
        Buffer.from('\n'),
        create_buffer_page_with_annotation_1.default(pdf, info, pageRef, widget),
    ]);
    pdf = Buffer.concat([pdf, Buffer.from('\n'), create_buffer_trailer_1.default(pdf, info, addedReferences)]);
    return pdf;
};
var getAssembledPdf = function (pdf, index, input, stream) {
    var finalPdf = pdf;
    finalPdf = Buffer.concat([
        finalPdf,
        Buffer.from('\n'),
        Buffer.from(index + " 0 obj\n"),
        Buffer.from(pdfobject_1.PDFObject.convert(input)),
    ]);
    if (stream) {
        finalPdf = Buffer.concat([
            finalPdf,
            Buffer.from('\nstream\n'),
            Buffer.from(stream),
            Buffer.from('\nendstream'),
        ]);
    }
    finalPdf = Buffer.concat([finalPdf, Buffer.from('\nendobj\n')]);
    return finalPdf;
};
var isContainBufferRootWithAcrofrom = function (pdf) {
    var bufferRootWithAcroformRefRegex = new RegExp('\\/AcroForm\\s+(\\d+\\s\\d+\\sR)', 'g');
    var match = bufferRootWithAcroformRefRegex.exec(pdf.toString());
    return match != null && match[1] != null && match[1] !== '';
};
exports.default = plainAddPlaceholder;
